@@ -92,6 +92,7 @@ struct a2dp_setup {
 	struct avdtp_remote_sep *rsep;
 	struct avdtp_stream *stream;
 	struct avdtp_error *err;
+	avdtp_set_configuration_cb setconf_cb;
 	GSList *client_caps;
 	gboolean reconfigure;
 	gboolean start;
@@ -180,7 +181,7 @@ static gboolean finalize_config_errno(struct a2dp_setup *s, int err)
 {
 	struct avdtp_error avdtp_err;
 
-	avdtp_error_init(&avdtp_err, AVDTP_ERROR_ERRNO, -err);
+	avdtp_error_init(&avdtp_err, AVDTP_ERRNO, -err);
 	s->err = err ? &avdtp_err : NULL;
 
 	return finalize_config(s);
@@ -230,7 +231,7 @@ static gboolean finalize_suspend_errno(struct a2dp_setup *s, int err)
 {
 	struct avdtp_error avdtp_err;
 
-	avdtp_error_init(&avdtp_err, AVDTP_ERROR_ERRNO, -err);
+	avdtp_error_init(&avdtp_err, AVDTP_ERRNO, -err);
 	s->err = err ? &avdtp_err : NULL;
 
 	return finalize_suspend(s);
@@ -311,14 +312,50 @@ static void stream_state_changed(struct avdtp_stream *stream,
 
 }
 
+static gboolean auto_config(gpointer data)
+{
+	struct a2dp_setup *setup = data;
+	struct avdtp_error *err = NULL;
+
+	/* Check if configuration was aborted */
+	if (setup->sep->stream == NULL)
+		return FALSE;
+
+	if (setup->err != NULL) {
+		err = setup->err;
+		goto done;
+	}
+
+	avdtp_stream_add_cb(setup->session, setup->stream,
+				stream_state_changed, setup->sep);
+
+	if (setup->sep->type == AVDTP_SEP_TYPE_SOURCE)
+		sink_new_stream(setup->dev, setup->session, setup->stream);
+	else
+		source_new_stream(setup->dev, setup->session, setup->stream);
+
+done:
+	if (setup->setconf_cb)
+		setup->setconf_cb(setup->session, setup->stream, setup->err);
+
+	finalize_config(setup);
+
+	if (err)
+		g_free(err);
+
+	return FALSE;
+}
+
 static gboolean sbc_setconf_ind(struct avdtp *session,
-				struct avdtp_local_sep *sep,
-				struct avdtp_stream *stream,
-				GSList *caps, uint8_t *err,
-				uint8_t *category, void *user_data)
+					struct avdtp_local_sep *sep,
+					struct avdtp_stream *stream,
+					GSList *caps,
+					avdtp_set_configuration_cb cb,
+					void *user_data)
 {
 	struct a2dp_sep *a2dp_sep = user_data;
 	struct audio_device *dev;
+	struct a2dp_setup *setup;
 
 	if (a2dp_sep->type == AVDTP_SEP_TYPE_SINK)
 		DBG("Sink %p: Set_Configuration_Ind", sep);
@@ -326,12 +363,23 @@ static gboolean sbc_setconf_ind(struct avdtp *session,
 		DBG("Source %p: Set_Configuration_Ind", sep);
 
 	dev = a2dp_get_dev(session);
-	if (!dev) {
-		*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-		*category = 0x00;
+	if (!dev)
 		return FALSE;
+
+	setup = find_setup_by_session(session);
+	if (!setup) {
+		setup = g_new0(struct a2dp_setup, 1);
+		setup->session = avdtp_ref(session);
+		setup->dev = dev;
+		setups = g_slist_append(setups, setup);
 	}
 
+	setup_ref(setup);
+	a2dp_sep->stream = stream;
+	setup->sep = a2dp_sep;
+	setup->stream = stream;
+	setup->setconf_cb = cb;
+
 	/* Check valid settings */
 	for (; caps != NULL; caps = g_slist_next(caps)) {
 		struct avdtp_service_capability *cap = caps->data;
@@ -340,9 +388,10 @@ static gboolean sbc_setconf_ind(struct avdtp *session,
 
 		if (cap->category == AVDTP_DELAY_REPORTING &&
 					!a2dp_sep->delay_reporting) {
-			*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-			*category = AVDTP_DELAY_REPORTING;
-			return FALSE;
+			setup->err = g_new(struct avdtp_error, 1);
+			avdtp_error_init(setup->err, AVDTP_DELAY_REPORTING,
+						AVDTP_UNSUPPORTED_CONFIGURATION);
+			goto done;
 		}
 
 		if (cap->category != AVDTP_MEDIA_CODEC)
@@ -360,18 +409,15 @@ static gboolean sbc_setconf_ind(struct avdtp *session,
 
 		if (sbc_cap->min_bitpool < MIN_BITPOOL ||
 					sbc_cap->max_bitpool > MAX_BITPOOL) {
-			*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-			*category = AVDTP_MEDIA_CODEC;
-			return FALSE;
+			setup->err = g_new(struct avdtp_error, 1);
+			avdtp_error_init(setup->err, AVDTP_MEDIA_CODEC,
+					AVDTP_UNSUPPORTED_CONFIGURATION);
+			goto done;
 		}
 	}
 
-	avdtp_stream_add_cb(session, stream, stream_state_changed, a2dp_sep);
-	a2dp_sep->stream = stream;
-
-	if (a2dp_sep->type == AVDTP_SEP_TYPE_SOURCE)
-		sink_new_stream(dev, session, stream);
-
+done:
+	g_idle_add(auto_config, setup);
 	return TRUE;
 }
 
@@ -439,13 +485,15 @@ static gboolean sbc_getcap_ind(struct avdtp *session, struct avdtp_local_sep *se
 }
 
 static gboolean mpeg_setconf_ind(struct avdtp *session,
-				struct avdtp_local_sep *sep,
-				struct avdtp_stream *stream,
-				GSList *caps, uint8_t *err,
-				uint8_t *category, void *user_data)
+					struct avdtp_local_sep *sep,
+					struct avdtp_stream *stream,
+					GSList *caps,
+					avdtp_set_configuration_cb cb,
+					void *user_data)
 {
 	struct a2dp_sep *a2dp_sep = user_data;
 	struct audio_device *dev;
+	struct a2dp_setup *setup;
 
 	if (a2dp_sep->type == AVDTP_SEP_TYPE_SINK)
 		DBG("Sink %p: Set_Configuration_Ind", sep);
@@ -453,29 +501,37 @@ static gboolean mpeg_setconf_ind(struct avdtp *session,
 		DBG("Source %p: Set_Configuration_Ind", sep);
 
 	dev = a2dp_get_dev(session);
-	if (!dev) {
-		*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-		*category = 0x00;
+	if (!dev)
 		return FALSE;
+
+	setup = find_setup_by_session(session);
+	if (!setup) {
+		setup = g_new0(struct a2dp_setup, 1);
+		setup->session = avdtp_ref(session);
+		setup->dev = dev;
+		setups = g_slist_append(setups, setup);
 	}
 
+	setup_ref(setup);
+	a2dp_sep->stream = stream;
+	setup->sep = a2dp_sep;
+	setup->stream = stream;
+	setup->setconf_cb = cb;
+
 	for (; caps != NULL; caps = g_slist_next(caps)) {
 		struct avdtp_service_capability *cap = caps->data;
 
 		if (cap->category == AVDTP_DELAY_REPORTING &&
 					!a2dp_sep->delay_reporting) {
-			*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-			*category = AVDTP_DELAY_REPORTING;
-			return FALSE;
+			setup->err = g_new(struct avdtp_error, 1);
+			avdtp_error_init(setup->err, AVDTP_DELAY_REPORTING,
+					AVDTP_UNSUPPORTED_CONFIGURATION);
+			goto done;
 		}
 	}
 
-	avdtp_stream_add_cb(session, stream, stream_state_changed, a2dp_sep);
-	a2dp_sep->stream = stream;
-
-	if (a2dp_sep->type == AVDTP_SEP_TYPE_SOURCE)
-		sink_new_stream(dev, session, stream);
-
+done:
+	g_idle_add(auto_config, setup);
 	return TRUE;
 }
 
@@ -536,14 +592,30 @@ static gboolean mpeg_getcap_ind(struct avdtp *session,
 	return TRUE;
 }
 
+static void endpoint_setconf_cb(struct media_endpoint *endpoint, void *ret,
+						int size, void *user_data)
+{
+	struct a2dp_setup *setup = user_data;
+
+	if (ret == NULL) {
+		setup->err = g_new(struct avdtp_error, 1);
+		avdtp_error_init(setup->err, AVDTP_MEDIA_CODEC,
+					AVDTP_UNSUPPORTED_CONFIGURATION);
+	}
+
+	auto_config(setup);
+}
+
 static gboolean endpoint_setconf_ind(struct avdtp *session,
-					struct avdtp_local_sep *sep,
-					struct avdtp_stream *stream,
-					GSList *caps, uint8_t *err,
-					uint8_t *category, void *user_data)
+						struct avdtp_local_sep *sep,
+						struct avdtp_stream *stream,
+						GSList *caps,
+						avdtp_set_configuration_cb cb,
+						void *user_data)
 {
 	struct a2dp_sep *a2dp_sep = user_data;
 	struct audio_device *dev;
+	struct a2dp_setup *setup;
 
 	if (a2dp_sep->type == AVDTP_SEP_TYPE_SINK)
 		DBG("Sink %p: Set_Configuration_Ind", sep);
@@ -551,12 +623,23 @@ static gboolean endpoint_setconf_ind(struct avdtp *session,
 		DBG("Source %p: Set_Configuration_Ind", sep);
 
 	dev = a2dp_get_dev(session);
-	if (!dev) {
-		*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-		*category = 0x00;
+	if (!dev)
 		return FALSE;
+
+	setup = find_setup_by_session(session);
+	if (!setup) {
+		setup = g_new0(struct a2dp_setup, 1);
+		setup->session = avdtp_ref(session);
+		setup->dev = dev;
+		setups = g_slist_append(setups, setup);
 	}
 
+	setup_ref(setup);
+	a2dp_sep->stream = stream;
+	setup->sep = a2dp_sep;
+	setup->stream = stream;
+	setup->setconf_cb = cb;
+
 	for (; caps != NULL; caps = g_slist_next(caps)) {
 		struct avdtp_service_capability *cap = caps->data;
 		struct avdtp_media_codec_capability *codec;
@@ -564,9 +647,10 @@ static gboolean endpoint_setconf_ind(struct avdtp *session,
 
 		if (cap->category == AVDTP_DELAY_REPORTING &&
 					!a2dp_sep->delay_reporting) {
-			*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-			*category = AVDTP_DELAY_REPORTING;
-			return FALSE;
+			setup->err = g_new(struct avdtp_error, 1);
+			avdtp_error_init(setup->err, AVDTP_DELAY_REPORTING,
+					AVDTP_UNSUPPORTED_CONFIGURATION);
+			goto done;
 		}
 
 		if (cap->category != AVDTP_MEDIA_CODEC)
@@ -575,28 +659,25 @@ static gboolean endpoint_setconf_ind(struct avdtp *session,
 		codec = (struct avdtp_media_codec_capability *) cap->data;
 
 		if (codec->media_codec_type != a2dp_sep->codec) {
-			*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-			*category = AVDTP_MEDIA_CODEC;
-			return FALSE;
+			setup->err = g_new(struct avdtp_error, 1);
+			avdtp_error_init(setup->err, AVDTP_MEDIA_CODEC,
+					AVDTP_UNSUPPORTED_CONFIGURATION);
+			goto done;
 		}
 
 		ret = media_endpoint_set_configuration(a2dp_sep->endpoint, dev,
 					codec->data, cap->length - sizeof(*codec),
-					NULL, NULL);
+					endpoint_setconf_cb, setup);
 		if (ret)
-			break;
+			return TRUE;
 
-		*err = AVDTP_UNSUPPORTED_CONFIGURATION;
-		*category = AVDTP_MEDIA_CODEC;
-		return FALSE;
+		avdtp_error_init(setup->err, AVDTP_MEDIA_CODEC,
+					AVDTP_UNSUPPORTED_CONFIGURATION);
+		break;
 	}
 
-	avdtp_stream_add_cb(session, stream, stream_state_changed, a2dp_sep);
-	a2dp_sep->stream = stream;
-
-	if (a2dp_sep->type == AVDTP_SEP_TYPE_SOURCE)
-		sink_new_stream(dev, session, stream);
-
+done:
+	g_idle_add(auto_config, setup);
 	return TRUE;
 }
 
@@ -647,6 +728,27 @@ static gboolean endpoint_getcap_ind(struct avdtp *session,
 	return TRUE;
 }
 
+static void endpoint_open_cb(struct media_endpoint *endpoint, void *ret,
+						int size, void *user_data)
+{
+	struct a2dp_setup *setup = user_data;
+	int err;
+
+	if (ret == NULL) {
+		setup->stream = NULL;
+		finalize_config_errno(setup, -EPERM);
+		return;
+	}
+
+	err = avdtp_open(setup->session, setup->stream);
+	if (err == 0)
+		return;
+
+	error("Error on avdtp_open %s (%d)", strerror(-err), -err);
+	setup->stream = NULL;
+	finalize_config_errno(setup, err);
+}
+
 static void setconf_cfm(struct avdtp *session, struct avdtp_local_sep *sep,
 				struct avdtp_stream *stream,
 				struct avdtp_error *err, void *user_data)
@@ -695,11 +797,14 @@ static void setconf_cfm(struct avdtp *session, struct avdtp_local_sep *sep,
 
 		if (media_endpoint_set_configuration(a2dp_sep->endpoint, dev,
 						codec->data, service->length -
-						sizeof(*codec), NULL, NULL) ==
-						FALSE) {
-			setup->stream = NULL;
-			finalize_config_errno(setup, -EPERM);
-		}
+						sizeof(*codec),
+						endpoint_open_cb, setup) ==
+						TRUE)
+			return;
+
+		setup->stream = NULL;
+		finalize_config_errno(setup, -EPERM);
+		return;
 	}
 
 	ret = avdtp_open(session, stream);
@@ -897,7 +1002,7 @@ static void suspend_cfm(struct avdtp *session, struct avdtp_local_sep *sep,
 	} else if (avdtp_start(session, a2dp_sep->stream) < 0) {
 		struct avdtp_error start_err;
 		error("avdtp_start failed");
-		avdtp_error_init(&start_err, AVDTP_ERROR_ERRNO, EIO);
+		avdtp_error_init(&start_err, AVDTP_ERRNO, EIO);
 		setup->err = err;
 		finalize_suspend(setup);
 	}
@@ -1226,7 +1331,8 @@ int a2dp_register(DBusConnection *conn, const bdaddr_t *src, GKeyFile *config)
 {
 	int sbc_srcs = 1, sbc_sinks = 1;
 	int mpeg12_srcs = 0, mpeg12_sinks = 0;
-	gboolean source = TRUE, sink = FALSE, socket = TRUE, delay_reporting;
+	gboolean source = TRUE, sink = FALSE, socket = TRUE;
+	gboolean delay_reporting = FALSE;
 	char *str;
 	GError *err = NULL;
 	int i;
@@ -1330,8 +1436,10 @@ proceed:
 		servers = g_slist_append(servers, server);
 	}
 
-	delay_reporting = g_key_file_get_boolean(config, "A2DP",
+	if (config)
+		delay_reporting = g_key_file_get_boolean(config, "A2DP",
 						"DelayReporting", NULL);
+
 	if (delay_reporting)
 		server->version = 0x0103;
 	else
