@@ -129,6 +129,17 @@ static void media_endpoint_exit(DBusConnection *connection, void *user_data)
 	media_endpoint_remove(endpoint);
 }
 
+static void headset_setconf_cb(struct media_endpoint *endpoint, void *ret,
+						int size, void *user_data)
+{
+	struct audio_device *dev = user_data;
+
+	if (ret != NULL)
+		return;
+
+	headset_set_state(dev, HEADSET_STATE_DISCONNECTED);
+}
+
 static void headset_state_changed(struct audio_device *dev,
 					headset_state_t old_state,
 					headset_state_t new_state,
@@ -148,7 +159,8 @@ static void headset_state_changed(struct audio_device *dev,
 		if (old_state != HEADSET_STATE_PLAY_IN_PROGRESS &&
 				old_state != HEADSET_STATE_PLAYING)
 			media_endpoint_set_configuration(endpoint, dev, NULL,
-								0, NULL, NULL);
+							0, headset_setconf_cb,
+									dev);
 		break;
 	case HEADSET_STATE_PLAY_IN_PROGRESS:
 		break;
@@ -441,6 +453,9 @@ static void endpoint_reply(DBusPendingCall *call, void *user_data)
 
 		/* Clear endpoint configuration in case of NO_REPLY error */
 		if (dbus_error_has_name(&err, DBUS_ERROR_NO_REPLY)) {
+			if (request->cb)
+				request->cb(endpoint, NULL, size,
+							request->user_data);
 			media_endpoint_clear_configuration(endpoint);
 			dbus_message_unref(reply);
 			dbus_error_free(&err);
@@ -478,7 +493,8 @@ static void endpoint_reply(DBusPendingCall *call, void *user_data)
 done:
 	dbus_message_unref(reply);
 
-	request->cb(endpoint, ret, size, request->user_data);
+	if (request->cb)
+		request->cb(endpoint, ret, size, request->user_data);
 
 	endpoint_request_free(request);
 	endpoint->request = NULL;
@@ -526,8 +542,7 @@ gboolean media_endpoint_set_configuration(struct media_endpoint *endpoint,
 					void *user_data)
 {
 	DBusConnection *conn;
-	DBusMessage *msg, *reply;
-	DBusError err;
+	DBusMessage *msg;
 	const char *path;
 	DBusMessageIter iter;
 
@@ -556,36 +571,7 @@ gboolean media_endpoint_set_configuration(struct media_endpoint *endpoint,
 
 	transport_get_properties(endpoint->transport, &iter);
 
-	if (cb != NULL)
-		return media_endpoint_async_call(conn, msg, endpoint, cb, user_data);
-
-	dbus_error_init(&err);
-
-	DBG("Calling %s: name = %s path = %s", dbus_message_get_member(msg),
-			dbus_message_get_destination(msg),
-			dbus_message_get_path(msg));
-
-	/* FIXME: remove once we can reply setconf asynchronously */
-	reply = dbus_connection_send_with_reply_and_block(conn, msg,
-							REQUEST_TIMEOUT, &err);
-
-	dbus_message_unref(msg);
-
-	if (reply) {
-		dbus_message_unref(reply);
-		return TRUE;
-	}
-
-	if (dbus_error_is_set(&err)) {
-		error("Endpoint replied with an error: %s", err.name);
-
-		if (dbus_error_has_name(&err, DBUS_ERROR_NO_REPLY))
-			media_endpoint_clear_configuration(endpoint);
-
-		dbus_error_free(&err);
-	}
-
-	return FALSE;
+	return media_endpoint_async_call(conn, msg, endpoint, cb, user_data);
 }
 
 gboolean media_endpoint_select_configuration(struct media_endpoint *endpoint,
