@@ -2000,7 +2000,8 @@ static int adapter_setup(struct btd_adapter *adapter, const char *mode)
 		adapter_ops->read_inq_tx_pwr(adapter->dev_id);
 
 	if (dev->features[4] & LMP_LE) {
-		err = adapter_ops->write_le_host(adapter->dev_id, 0x01, 0x00);
+		uint8_t simul = (dev->features[6] & LMP_LE_BREDR) ? 0x01 : 0x00;
+		err = adapter_ops->write_le_host(adapter->dev_id, 0x01, simul);
 		if (err < 0) {
 			error("Can't write LE host supported for %s: %s(%d)",
 					adapter->path, strerror(-err), -err);
@@ -3209,7 +3210,6 @@ void adapter_remove_connection(struct btd_adapter *adapter,
 
 	/* clean pending HCI cmds */
 	device_get_address(device, &bdaddr);
-	hci_req_queue_remove(adapter->dev_id, &bdaddr);
 
 	if (device_is_authenticating(device))
 		device_cancel_authentication(device, TRUE);
@@ -3649,3 +3649,15 @@ void btd_adapter_update_local_ext_features(struct btd_adapter *adapter,
 
 	memcpy(dev->extfeatures, features, 8);
 }
+
+int btd_adapter_get_remote_name(struct btd_adapter *adapter, bdaddr_t *bdaddr)
+{
+	return adapter_ops->resolve_name(adapter->dev_id, bdaddr);
+}
+
+int btd_adapter_get_remote_version(struct btd_adapter *adapter,
+					uint16_t handle, gboolean delayed)
+{
+	return adapter_ops->get_remote_version(adapter->dev_id, handle,
+								delayed);
+}
