@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -47,10 +47,11 @@
 #include <dbus/dbus.h>
 
 #include "hcid.h"
-#include "logging.h"
+#include "log.h"
 #include "textfile.h"
 
 #include "adapter.h"
+#include "device.h"
 #include "dbus-hci.h"
 #include "storage.h"
 #include "manager.h"
@@ -79,137 +80,6 @@ struct g_io_info {
 
 static struct g_io_info io_data[HCI_MAX_DEV];
 
-static GSList *hci_req_queue = NULL;
-
-static struct hci_req_data *hci_req_data_new(int dev_id, const bdaddr_t *dba,
-					uint16_t ogf, uint16_t ocf, int event,
-					const void *cparam, int clen)
-{
-	struct hci_req_data *data;
-
-	data = g_new0(struct hci_req_data, 1);
-
-	data->cparam = g_malloc(clen);
-	memcpy(data->cparam, cparam, clen);
-
-	bacpy(&data->dba, dba);
-
-	data->dev_id = dev_id;
-	data->status = REQ_PENDING;
-	data->ogf    = ogf;
-	data->ocf    = ocf;
-	data->event  = event;
-	data->clen   = clen;
-
-	return data;
-}
-
-static int hci_req_find_by_devid(const void *data, const void *user_data)
-{
-	const struct hci_req_data *req = data;
-	const int *dev_id = user_data;
-
-	return (*dev_id - req->dev_id);
-}
-
-static void hci_req_queue_process(int dev_id)
-{
-	int dd, ret_val;
-
-	/* send the next pending cmd */
-	dd = hci_open_dev(dev_id);
-	if (dd < 0) {
-		error("hci_open_dev(%d): %s (%d)", dev_id, strerror(errno),
-									errno);
-		return;
-	}
-
-	do {
-		struct hci_req_data *data;
-		GSList *l = g_slist_find_custom(hci_req_queue, &dev_id, hci_req_find_by_devid);
-
-		if (!l)
-			break;
-
-		data = l->data;
-		data->status = REQ_SENT;
-
-		ret_val = hci_send_cmd(dd, data->ogf, data->ocf, data->clen, data->cparam);
-		if (ret_val < 0) {
-			hci_req_queue = g_slist_remove(hci_req_queue, data);
-			g_free(data->cparam);
-			g_free(data);
-		}
-
-	} while (ret_val < 0);
-
-	hci_close_dev(dd);
-}
-
-static void hci_req_queue_append(struct hci_req_data *data)
-{
-	GSList *l;
-	struct hci_req_data *match;
-
-
-	hci_req_queue = g_slist_append(hci_req_queue, data);
-
-	l = g_slist_find_custom(hci_req_queue, &data->dev_id, hci_req_find_by_devid);
-	match = l->data;
-
-	if (match->status == REQ_SENT)
-		return;
-
-	hci_req_queue_process(data->dev_id);
-}
-
-void hci_req_queue_remove(int dev_id, bdaddr_t *dba)
-{
-	GSList *cur, *next;
-	struct hci_req_data *req;
-
-	for (cur = hci_req_queue; cur != NULL; cur = next) {
-		req = cur->data;
-		next = cur->next;
-		if ((req->dev_id != dev_id) || (bacmp(&req->dba, dba)))
-			continue;
-
-		hci_req_queue = g_slist_remove(hci_req_queue, req);
-		g_free(req->cparam);
-		g_free(req);
-	}
-}
-
-static void check_pending_hci_req(int dev_id, int event)
-{
-	struct hci_req_data *data;
-	GSList *l;
-
-	if (!hci_req_queue)
-		return;
-
-	/* find the first element(pending)*/
-	l = g_slist_find_custom(hci_req_queue, &dev_id, hci_req_find_by_devid);
-
-	if (!l)
-		return;
-
-	data = l->data;
-
-	/* skip if there is pending confirmation */
-	if (data->status == REQ_SENT) {
-		if (data->event != event)
-			return;
-
-		/* remove the confirmed cmd */
-		hci_req_queue = g_slist_remove(hci_req_queue, data);
-		g_free(data->cparam);
-		g_free(data);
-	}
-
-	hci_req_queue_process(dev_id);
-}
-
 static int get_handle(int dev, bdaddr_t *sba, bdaddr_t *dba, uint16_t *handle)
 {
 	struct hci_conn_list_req *cl;
@@ -299,12 +169,17 @@ static inline void update_lastused(bdaddr_t *sba, bdaddr_t *dba)
 
 static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 {
+	struct btd_adapter *adapter;
+	struct btd_device *device;
 	struct hci_auth_info_req req;
 	unsigned char key[16];
 	char sa[18], da[18];
 	uint8_t type;
 	int err;
 
+	if (!get_adapter_and_device(sba, dba, &adapter, &device, FALSE))
+		device = NULL;
+
 	ba2str(sba, sa); ba2str(dba, da);
 	info("link_key_request (sba=%s, dba=%s)", sa, da);
 
@@ -314,33 +189,37 @@ static void link_key_request(int dev, bdaddr_t *sba, bdaddr_t *dba)
 	err = ioctl(dev, HCIGETAUTHINFO, (unsigned long) &req);
 	if (err < 0) {
 		if (errno != EINVAL)
-			debug("HCIGETAUTHINFO failed %s (%d)",
+			DBG("HCIGETAUTHINFO failed %s (%d)",
 						strerror(errno), errno);
 		req.type = 0x00;
 	}
 
-	debug("kernel auth requirements = 0x%02x", req.type);
+	DBG("kernel auth requirements = 0x%02x", req.type);
 
-	err = read_link_key(sba, dba, key, &type);
-	if (err < 0) {
+	if (main_opts.debug_keys && device && device_get_debug_key(device, key))
+		type = 0x03;
+	else if (read_link_key(sba, dba, key, &type) < 0 || type == 0x03) {
 		/* Link key not found */
 		hci_send_cmd(dev, OGF_LINK_CTL, OCF_LINK_KEY_NEG_REPLY, 6, dba);
-	} else {
-		/* Link key found */
+		return;
+	}
+
+	/* Link key found */
+
+	DBG("link key type = 0x%02x", type);
+
+	/* Don't use unauthenticated combination keys if MITM is
+	 * required */
+	if (type == 0x04 && req.type != 0xff && (req.type & 0x01))
+		hci_send_cmd(dev, OGF_LINK_CTL, OCF_LINK_KEY_NEG_REPLY,
+								6, dba);
+	else {
 		link_key_reply_cp lr;
+
 		memcpy(lr.link_key, key, 16);
 		bacpy(&lr.bdaddr, dba);
 
-		debug("stored link key type = 0x%02x", type);
-
-		/* Don't use debug link keys (0x03) and also don't use
-		 * unauthenticated combination keys if MITM is required */
-		if (type == 0x03 || (type == 0x04 && req.type != 0xff &&
-							(req.type & 0x01)))
-			hci_send_cmd(dev, OGF_LINK_CTL,
-					OCF_LINK_KEY_NEG_REPLY, 6, dba);
-		else
-			hci_send_cmd(dev, OGF_LINK_CTL, OCF_LINK_KEY_REPLY,
+		hci_send_cmd(dev, OGF_LINK_CTL, OCF_LINK_KEY_REPLY,
 						LINK_KEY_REPLY_CP_SIZE, &lr);
 	}
 }
@@ -365,11 +244,14 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 	dev_id = hci_devid(sa);
 	if (dev_id < 0)
 		err = -errno;
-	else
+	else {
 		err = hcid_dbus_link_key_notify(sba, dba, evt->link_key,
 						evt->key_type,
 						io_data[dev_id].pin_length,
 						old_key_type);
+		io_data[dev_id].pin_length = -1;
+	}
+
 	if (err < 0) {
 		uint16_t handle;
 
@@ -391,8 +273,6 @@ static void link_key_notify(int dev, bdaddr_t *sba, void *ptr)
 						DISCONNECT_CP_SIZE, &cp);
 		}
 	}
-
-	io_data[dev_id].pin_length = -1;
 }
 
 static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
@@ -413,6 +293,8 @@ static void return_link_keys(int dev, bdaddr_t *sba, void *ptr)
 
 		info("return_link_keys (sba=%s, dba=%s)", sa, da);
 
+		hcid_dbus_returned_link_key(sba, &dba);
+
 		ptr += 22;
 	}
 }
@@ -571,28 +453,12 @@ static void start_inquiry(bdaddr_t *local, uint8_t status, gboolean periodic)
 
 	state = adapter_get_state(adapter);
 
-	/* Disable name resolution for non D-Bus clients */
-	if (!adapter_has_discov_sessions(adapter))
-		state &= ~RESOLVE_NAME;
+	if (periodic)
+		state |= STATE_PINQ;
+	else
+		state |= STATE_STDINQ;
 
-	if (periodic) {
-		state |= PERIODIC_INQUIRY;
-		adapter_set_state(adapter, state);
-		return;
-	}
-
-	state |= STD_INQUIRY;
 	adapter_set_state(adapter, state);
-
-	/*
-	 * Cancel pending remote name request and clean the device list
-	 * when inquiry is supported in periodic inquiry idle state.
-	 */
-	if (adapter_get_state(adapter) & PERIODIC_INQUIRY) {
-		pending_remote_name_cancel(adapter);
-
-		clear_found_devices_list(adapter);
-	}
 }
 
 static void inquiry_complete(bdaddr_t *local, uint8_t status, gboolean periodic)
@@ -612,43 +478,58 @@ static void inquiry_complete(bdaddr_t *local, uint8_t status, gboolean periodic)
 		return;
 	}
 
-	/*
-	 * The following scenarios can happen:
-	 * 1. standard inquiry: always send discovery completed signal
-	 * 2. standard inquiry + name resolving: send discovery completed
-	 *    after name resolving
-	 * 3. periodic inquiry: skip discovery completed signal
-	 * 4. periodic inquiry + standard inquiry: always send discovery
-	 *    completed signal
-	 *
-	 * Keep in mind that non D-Bus requests can arrive.
-	 */
-	if (periodic) {
-		state = adapter_get_state(adapter);
-		state &= ~PERIODIC_INQUIRY;
-		adapter_set_state(adapter, state);
-		return;
-	}
-
-	if (adapter_resolve_names(adapter) == 0)
-		return;
-
 	state = adapter_get_state(adapter);
-	/*
-	 * workaround to identify situation when there is no devices around
-	 * but periodic inquiry is active.
-	 */
-	if (!(state & STD_INQUIRY) && !(state & PERIODIC_INQUIRY)) {
-		state |= PERIODIC_INQUIRY;
-		adapter_set_state(adapter, state);
+	state &= ~(STATE_STDINQ | STATE_PINQ);
+	adapter_set_state(adapter, state);
+}
+
+static inline void remote_features_notify(int dev, bdaddr_t *sba, void *ptr)
+{
+	evt_remote_host_features_notify *evt = ptr;
+
+	if (evt->features[0] & 0x01)
+		hcid_dbus_set_legacy_pairing(sba, &evt->bdaddr, FALSE);
+	else
+		hcid_dbus_set_legacy_pairing(sba, &evt->bdaddr, TRUE);
+
+	write_features_info(sba, &evt->bdaddr, NULL, evt->features);
+}
+
+static void write_le_host_complete(bdaddr_t *sba, uint8_t status)
+{
+	struct btd_adapter *adapter;
+
+	if (status)
+		return;
+
+	adapter = manager_find_adapter(sba);
+	if (!adapter) {
+		error("No matching adapter found");
 		return;
 	}
 
-	/* reset the discover type to be able to handle D-Bus and non D-Bus
-	 * requests */
-	state &= ~STD_INQUIRY;
-	state &= ~PERIODIC_INQUIRY;
-	adapter_set_state(adapter, state);
+	btd_adapter_read_local_ext_features(adapter);
+}
+
+static void read_local_ext_features_complete(bdaddr_t *sba,
+				const read_local_ext_features_rp *rp)
+{
+	struct btd_adapter *adapter;
+
+	if (rp->status)
+		return;
+
+	adapter = manager_find_adapter(sba);
+	if (!adapter) {
+		error("No matching adapter found");
+		return;
+	}
+
+	/* Local Extended feature page number is 1 */
+	if (rp->page_num != 1)
+		return;
+
+	btd_adapter_update_local_ext_features(adapter, rp->features);
 }
 
 static inline void cmd_status(int dev, bdaddr_t *sba, void *ptr)
@@ -660,6 +541,21 @@ static inline void cmd_status(int dev, bdaddr_t *sba, void *ptr)
 		start_inquiry(sba, evt->status, FALSE);
 }
 
+static void read_scan_complete(bdaddr_t *sba, uint8_t status, void *ptr)
+{
+	struct btd_adapter *adapter;
+	read_scan_enable_rp *rp = ptr;
+
+	adapter = manager_find_adapter(sba);
+
+	if (!adapter) {
+		error("Unable to find matching adapter");
+		return;
+	}
+
+	adapter_mode_changed(adapter, rp->enable);
+}
+
 static inline void cmd_complete(int dev, bdaddr_t *sba, void *ptr)
 {
 	evt_cmd_complete *evt = ptr;
@@ -667,6 +563,10 @@ static inline void cmd_complete(int dev, bdaddr_t *sba, void *ptr)
 	uint8_t status = *((uint8_t *) ptr + EVT_CMD_COMPLETE_SIZE);
 
 	switch (opcode) {
+	case cmd_opcode_pack(OGF_INFO_PARAM, OCF_READ_LOCAL_EXT_FEATURES):
+		ptr += sizeof(evt_cmd_complete);
+		read_local_ext_features_complete(sba, ptr);
+		break;
 	case cmd_opcode_pack(OGF_LINK_CTL, OCF_PERIODIC_INQUIRY):
 		start_inquiry(sba, status, TRUE);
 		break;
@@ -676,18 +576,40 @@ static inline void cmd_complete(int dev, bdaddr_t *sba, void *ptr)
 	case cmd_opcode_pack(OGF_LINK_CTL, OCF_INQUIRY_CANCEL):
 		inquiry_complete(sba, status, FALSE);
 		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_LE_HOST_SUPPORTED):
+		write_le_host_complete(sba, status);
+		break;
+	case cmd_opcode_pack(OGF_LE_CTL, OCF_LE_SET_SCAN_ENABLE):
+		hcid_dbus_le_set_scan_enable_complete(sba, status);
+		break;
 	case cmd_opcode_pack(OGF_HOST_CTL, OCF_CHANGE_LOCAL_NAME):
-		hcid_dbus_setname_complete(sba);
+		adapter_setname_complete(sba, status);
 		break;
 	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_SCAN_ENABLE):
 		hcid_dbus_setscan_enable_complete(sba);
 		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_READ_SCAN_ENABLE):
+		ptr += sizeof(evt_cmd_complete);
+		read_scan_complete(sba, status, ptr);
+		break;
 	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_CLASS_OF_DEV):
-		hcid_dbus_write_class_complete(sba);
+		adapter_set_class_complete(sba, status);
 		break;
 	case cmd_opcode_pack(OGF_HOST_CTL, OCF_WRITE_SIMPLE_PAIRING_MODE):
 		hcid_dbus_write_simple_pairing_mode_complete(sba);
 		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_READ_SIMPLE_PAIRING_MODE):
+		ptr += sizeof(evt_cmd_complete);
+		hcid_dbus_read_simple_pairing_mode_complete(sba, ptr);
+		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_READ_LOCAL_NAME):
+		ptr += sizeof(evt_cmd_complete);
+		adapter_update_local_name(sba, status, ptr);
+		break;
+	case cmd_opcode_pack(OGF_HOST_CTL, OCF_READ_INQ_RESPONSE_TX_POWER_LEVEL):
+		ptr += sizeof(evt_cmd_complete);
+		adapter_update_tx_power(sba, status, ptr);
+		break;
 	};
 }
 
@@ -747,7 +669,8 @@ static inline void inquiry_result(int dev, bdaddr_t *sba, int plen, void *ptr)
 	}
 }
 
-static inline void inquiry_result_with_rssi(int dev, bdaddr_t *sba, int plen, void *ptr)
+static inline void inquiry_result_with_rssi(int dev, bdaddr_t *sba,
+							int plen, void *ptr)
 {
 	uint8_t num = *(uint8_t *) ptr++;
 	int i;
@@ -786,7 +709,8 @@ static inline void inquiry_result_with_rssi(int dev, bdaddr_t *sba, int plen, vo
 	}
 }
 
-static inline void extended_inquiry_result(int dev, bdaddr_t *sba, int plen, void *ptr)
+static inline void extended_inquiry_result(int dev, bdaddr_t *sba,
+							int plen, void *ptr)
 {
 	uint8_t num = *(uint8_t *) ptr++;
 	int i;
@@ -806,7 +730,8 @@ static inline void extended_inquiry_result(int dev, bdaddr_t *sba, int plen, voi
 	}
 }
 
-static inline void remote_features_information(int dev, bdaddr_t *sba, void *ptr)
+static inline void remote_features_information(int dev, bdaddr_t *sba,
+								void *ptr)
 {
 	evt_read_remote_features_complete *evt = ptr;
 	bdaddr_t dba;
@@ -817,16 +742,21 @@ static inline void remote_features_information(int dev, bdaddr_t *sba, void *ptr
 	if (get_bdaddr(dev, sba, btohs(evt->handle), &dba) < 0)
 		return;
 
-	write_features_info(sba, &dba, evt->features);
+	write_features_info(sba, &dba, evt->features, NULL);
 }
 
 static inline void conn_complete(int dev, int dev_id, bdaddr_t *sba, void *ptr)
 {
 	evt_conn_complete *evt = ptr;
 	char filename[PATH_MAX];
-	remote_name_req_cp cp_name;
-	struct hci_req_data *data;
 	char local_addr[18], peer_addr[18], *str;
+	struct btd_adapter *adapter;
+
+	adapter = manager_find_adapter(sba);
+	if (!adapter) {
+		error("Unable to find matching adapter");
+		return;
+	}
 
 	if (evt->link_type != ACL_LINK)
 		return;
@@ -839,36 +769,20 @@ static inline void conn_complete(int dev, int dev_id, bdaddr_t *sba, void *ptr)
 
 	update_lastused(sba, &evt->bdaddr);
 
-	/* Request remote name */
-	memset(&cp_name, 0, sizeof(cp_name));
-	bacpy(&cp_name.bdaddr, &evt->bdaddr);
-	cp_name.pscan_rep_mode = 0x02;
-
-	data = hci_req_data_new(dev_id, &evt->bdaddr, OGF_LINK_CTL,
-				OCF_REMOTE_NAME_REQ, EVT_REMOTE_NAME_REQ_COMPLETE,
-				&cp_name, REMOTE_NAME_REQ_CP_SIZE);
-
-	hci_req_queue_append(data);
+	btd_adapter_get_remote_name(adapter, &evt->bdaddr);
 
 	/* check if the remote version needs be requested */
 	ba2str(sba, local_addr);
 	ba2str(&evt->bdaddr, peer_addr);
 
-	create_name(filename, sizeof(filename), STORAGEDIR, local_addr, "manufacturers");
+	create_name(filename, sizeof(filename), STORAGEDIR, local_addr,
+							"manufacturers");
 
 	str = textfile_get(filename, peer_addr);
-	if (!str) {
-		read_remote_version_cp cp;
-
-		memset(&cp, 0, sizeof(cp));
-		cp.handle = evt->handle;
-
-		data = hci_req_data_new(dev_id, &evt->bdaddr, OGF_LINK_CTL,
-					OCF_READ_REMOTE_VERSION, EVT_READ_REMOTE_VERSION_COMPLETE,
-					&cp, READ_REMOTE_VERSION_CP_SIZE);
-
-		hci_req_queue_append(data);
-	} else
+	if (!str)
+		btd_adapter_get_remote_version(adapter, btohs(evt->handle),
+									TRUE);
+	else
 		free(str);
 }
 
@@ -909,6 +823,28 @@ static inline void conn_request(int dev, bdaddr_t *sba, void *ptr)
 	write_remote_class(sba, &evt->bdaddr, class);
 }
 
+static inline void le_metaevent(int dev, bdaddr_t *sba, void *ptr)
+{
+	evt_le_meta_event *meta = ptr;
+	le_advertising_info *info;
+	uint8_t *rssi, num, i;
+
+	DBG("LE Meta Event");
+
+	if (meta->subevent != EVT_LE_ADVERTISING_REPORT)
+		return;
+
+	num = meta->data[0];
+	info = (le_advertising_info *) (meta->data + 1);
+
+	for (i = 0; i < num; i++) {
+		/* RSSI is last byte of the advertising report event */
+		rssi = info->data + info->length;
+		hcid_dbus_inquiry_result(sba, &info->bdaddr, 0, *rssi, NULL);
+		info = (le_advertising_info *) (rssi + 1);
+	}
+}
+
 static void delete_channel(GIOChannel *chan)
 {
 	int i;
@@ -923,7 +859,8 @@ static void delete_channel(GIOChannel *chan)
 	error("IO channel not found in the io_data table");
 }
 
-static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data)
+static gboolean io_security_event(GIOChannel *chan, GIOCondition cond,
+								gpointer data)
 {
 	unsigned char buf[HCI_MAX_EVENT_SIZE], *ptr = buf;
 	struct hci_dev_info *di = data;
@@ -957,7 +894,7 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 
 	ioctl(dev, HCIGETDEVINFO, (void *) di);
 
-	if (hci_test_bit(HCI_RAW, &di->flags))
+	if (ignore_device(di))
 		return TRUE;
 
 	switch (eh->evt) {
@@ -981,6 +918,10 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 		remote_features_information(dev, &di->bdaddr, ptr);
 		break;
 
+	case EVT_REMOTE_HOST_FEATURES_NOTIFY:
+		remote_features_notify(dev, &di->bdaddr, ptr);
+		break;
+
 	case EVT_INQUIRY_COMPLETE:
 		evt = (evt_cmd_status *) ptr;
 		inquiry_complete(&di->bdaddr, evt->status, FALSE);
@@ -1017,12 +958,9 @@ static gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer
 	case EVT_CONN_REQUEST:
 		conn_request(dev, &di->bdaddr, ptr);
 		break;
-	}
-
-	/* Check for pending command request */
-	check_pending_hci_req(di->dev_id, eh->evt);
-
-	switch (eh->evt) {
+	case EVT_LE_META_EVENT:
+		le_metaevent(dev, &di->bdaddr, ptr);
+		break;
 	case EVT_PIN_CODE_REQ:
 		pin_code_request(dev, &di->bdaddr, (bdaddr_t *) ptr);
 		break;
@@ -1115,6 +1053,7 @@ void start_security_manager(int hdev)
 	hci_filter_set_event(EVT_CONN_REQUEST, &flt);
 	hci_filter_set_event(EVT_CONN_COMPLETE, &flt);
 	hci_filter_set_event(EVT_DISCONN_COMPLETE, &flt);
+	hci_filter_set_event(EVT_LE_META_EVENT, &flt);
 	if (setsockopt(dev, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
 		error("Can't set filter on hci%d: %s (%d)",
 						hdev, strerror(errno), errno);
@@ -1133,13 +1072,13 @@ void start_security_manager(int hdev)
 
 	chan = g_io_channel_unix_new(dev);
 	g_io_channel_set_close_on_unref(chan, TRUE);
-	io_data[hdev].watch_id = g_io_add_watch_full(chan, G_PRIORITY_HIGH,
+	io_data[hdev].watch_id = g_io_add_watch_full(chan, G_PRIORITY_LOW,
 						G_IO_IN | G_IO_NVAL | G_IO_HUP | G_IO_ERR,
 						io_security_event, di, (GDestroyNotify) g_free);
 	io_data[hdev].channel = chan;
 	io_data[hdev].pin_length = -1;
 
-	if (hci_test_bit(HCI_RAW, &di->flags))
+	if (ignore_device(di))
 		return;
 
 	bacpy(&cp.bdaddr, BDADDR_ANY);
